function Cooccur_(fnCfg)

% Lexical Co_occurrence Dector for Contexual Map 
% Copyright (C) 2009 Xiaowei Zhao 

if nargin < 1
  disp(' ');
  disp(' Coocur_    Lexical Co-occurrence Detector that learns');
  disp(' to approximate transitional probabilities between words.');
  disp(' The resulting file (.prob.mat) is used for contexual map construction');
  disp(' If the halOut option is set to ON, the second resulting file (.mat)');
  disp(' is generated to save the representation obtained based on HAL method.');
  disp(' All simulation parameters are read from the configuration file.');
  disp(' ' );
  disp('  Usage: Cooccur_ fnCfg');
  disp(' ');
  return;
end

par = cooccur_read_par(fnCfg);
fprintf(1,'Reading parameters from %s...\n',fnCfg);
par

lex = read_strlist([par.datadir par.fnLex]);

fn = [par.datadir par.fnInd];
if ~isempty(findstr(par.fnInd,'.ind'))
  fprintf('Reading word indices...');
  fp = fopen(fn,'r');
  indices = fscanf(fp,'%d');
  fclose(fp);
else
  fprintf('Please provide indices file ended with .ind ...');
  return;
end

numInputs = length(indices);
fprintf(1,'numInputs=%d\n',numInputs);

maxWords = max(indices);
fprintf(1,'maxWords=%d\n',maxWords);
if length(par.lex) > 1
  fprintf(1,'Multiple vocabulary sizes specified.\n');
else 
  fprintf(1,'Lexicon: %d words\n',par.lex); 
end

if length(lex) < par.lex(end), error('Not enough labels available.'); end
if ~isempty(lex), fprintf(1,'Labels will be attached to word reps.\n'); end

%{
if maxWords > 3500
    fprintf(1,'Sparse matrix is used due to the huge lexicon size (above 3500) \n');
    probL = sparse(maxWords,maxWords);%zeros(maxWords,maxWords);
    probR = sparse(maxWords,maxWords);%zeros(maxWords,maxWords);
else
    probL = zeros(maxWords,maxWords);
    probR = zeros(maxWords,maxWords);
end
%}
indices(indices>par.lex(end))=0;
if par.lex > 3500
    fprintf(1,'Sparse matrix is used due to the huge lexicon size (above 3500) \n');
    probL = sparse(par.lex,par.lex);%zeros(maxWords,maxWords);
    probR = sparse(par.lex,par.lex);%zeros(maxWords,maxWords);
else
    probL = zeros(par.lex,par.lex);
    probR = zeros(par.lex,par.lex);
end


% training cycle
iword = 0;
disp(['Counting word co-occurrences for ' num2str(max(par.lex)) ' words...']);
iwordold = zeros(1,par.ctxlen);
unknown = 0;

for t = 1:numInputs
  
  if ~rem(t,10000) fprintf(1,'%7d',t); end
  if ~rem(t,100000) fprintf(1,'\n'); end
  
  % shift word indices in buffer
  for i = par.ctxlen:-1:2, iwordold(i) = iwordold(i-1); end
  iwordold(1) = iword;
    
  iword = indices(t);
  % if unknown word, skip to the next word
  if iword < 1
    unknown = unknown + 1; continue;
  end

  for i = 1:par.ctxlen
    if iwordold(i) > 0  % if known word
      delta = par.decay^(i-1);
      probL(iword,iwordold(i)) = probL(iword,iwordold(i)) + delta;
      probR(iwordold(i),iword) = probR(iwordold(i),iword) + delta;
    end
  end
  
end

fn = [par.datadir par.fnOut num2str(par.lex(end)) '.prob.mat'];
d = [probL probR];
probL = sparse(probL);
probR = sparse(probR);
save(fn,'probL', 'probR');

% Above is the code for getting the transitional probabilities
% below is the code for getting representation based on HAL; it only works
% when we have pal.halOut is ON

if strmatch(par.halOut, 'ON', 'exact')
    fprintf(1,'\nMaking word reps for all vocabulary sizes...');
    if nummatch(par.norm,[1 2])   
      normalized = 1;
      fprintf(1,' normalized on L%d\n',par.norm);
    else
      fprintf(1,' not normalized\n');
      normalized = 0;
    end

    for numW = par.lex
      fprintf(1,'Lexicon size: %d\n',numW);
      rep = sparse(numW,numW*2);  
      for iword = 1:numW
        if ~rem(iword,1000) fprintf(1,'%7d',iword); end
        if ~rem(iword,10000) fprintf(1,'\n'); end
        if normalized
          rep(iword,1:2:numW*2) = normalize(probL(iword,1:numW),par.norm);
          rep(iword,2:2:numW*2) = normalize(probR(iword,1:numW),par.norm);
        else
          rep(iword,1:2:numW*2) = probL(iword,1:numW);
          rep(iword,2:2:numW*2) = probR(iword,1:numW);
        end
      end

      fn = [par.datadir par.fnOut num2str(numW) '.mat'];

      % add labels if available
      if ~isempty(lex), repr.lab = lex(1:numW); else repr.lab = []; end
      repr.dat = full(rep(1:numW,:));

      % save file
      fprintf(1,'Saving output to %s...',fn);
      save(fn,'repr');
    end
end
